@@ -1,11 +1,13 @@
 /* App.m
  * Application class of Cenon
  *
- * Copyright (C) 1995-2009 by vhf interservice GmbH
+ * Copyright (C) 1995-2010 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1995-08-10
- * modified: 2009-06-24
+ * modified: 2010-04-10 (displayInfo: get version from plist and date from __DATE__)
+ *           2010-01-12 (take snapshot of open documents and their positions)
+ *           2009-06-24
  *           2009-03-27 (-sendEvent: ',' -> '.' on Apple)
  *           2009-02-25 (-listFromFile: check extensions for any case)
  *
@@ -319,6 +321,9 @@
 #endif
 
     [NSThread sleepUntilDate:[NSDate dateWithTimeIntervalSinceNow:500.0/1000000.0]];
+
+    /* Load Document Snapshot */
+    [self restoreSnapshot:self];
 }
 
 
@@ -552,9 +557,9 @@
 }
 
 - (BOOL)openFile:(NSString *)fileName
-{   id			list = 0;
-    NSString		*path;
-    NSFileManager	*fileManager = [NSFileManager defaultManager];
+{   id              list = 0;
+    NSString        *path;
+    NSFileManager   *fileManager = [NSFileManager defaultManager];
 
     if (!appIsRunning)	// give application a chance to load modules first
     {
@@ -606,7 +611,7 @@
 }
 
 /* Import files
- * modified: 20005-09-25
+ * modified: 2010-02-24 (allow extension ".tif")
  */
 typedef enum
 {   IMPORTTO_SELECTEDLAYER  = 0,
@@ -616,7 +621,7 @@ typedef enum
 - (void)import:sender
 {   NSArray     *fileTypes = [NSArray arrayWithObjects:@"hpgl", @"hgl", @"plt", @"ai",
                                               @"dxf", @"eps", @"ps", @"pdf", @"gerber", @"ger",
-                                              @"cenon", @"tiff", @"jpg", @"gif",
+                                              @"cenon", @"tiff", @"tif", @"jpg", @"jpeg", @"gif", @"png",
                                               @"font", @"pfa", @"din", @"drl", nil];
     NSString    *fileName;
     id          openPanel = [NSOpenPanel openPanel];
@@ -701,10 +706,10 @@ typedef enum
  * and loads the specified file into it.
  */
 - (void)openDocument:sender
-{   NSArray         *fileTypes = [NSArray arrayWithObjects:@"hpgl", @"hgl", @"plt", @"ai",
-                                              @"dxf", @"eps", @"ps", @"pdf", @"gerber", @"ger",
-                                              @"cenon", @"tiff", @"jpg", @"gif",
-                                              @"font", @"pfa", @"din", @"drl", nil];
+{   NSArray     *fileTypes = [NSArray arrayWithObjects:@"hpgl", @"hgl", @"plt", @"ai",
+                                  @"dxf", @"eps", @"ps", @"pdf", @"gerber", @"ger",
+                                  @"cenon", @"tiff", @"tif", @"jpg", @"jpeg", @"gif", @"png",
+                                  @"font", @"pfa", @"din", @"drl", nil];
     NSArray         *fileNames;
     id              openPanel = [NSOpenPanel openPanel];
     int             i, cnt;
@@ -850,6 +855,71 @@ typedef enum
     return YES;
 }
 
+/* Snapshots
+ * created:  2010-01-12
+ * modified: 2010-04-20
+ */
+- (void)takeSnapshot:sender
+{   NSUserDefaults  *defaults = [NSUserDefaults standardUserDefaults];
+    NSMutableArray  *snapArray = [NSMutableArray array];
+    int             count;
+
+    for ( count=[[self windows] count]-1; count>=0; count-- )
+    {	Document    *doc = [self documentInWindow:[[self windows] objectAtIndex:count]];
+
+        if (doc)
+        {   DocWindow   *window = [doc window];
+            NSString    *rString = propertyListFromNSRect([window frame]);
+            NSString    *sString = nil;
+#ifdef __APPLE__
+            if ( [window unfoldedHeight] > 20.0 )   // folded window -> save unfolded size
+                sString = propertyListFromFloat([window unfoldedHeight]);
+#endif
+            if (sString)
+                [snapArray addObject:[NSArray arrayWithObjects:[doc filename], rString, sString, nil]];
+            else
+                [snapArray addObject:[NSArray arrayWithObjects:[doc filename], rString, nil]];
+        }
+    }
+    [defaults setObject:snapArray forKey:@"snapShotDocuments"];
+}
+- (void)restoreSnapshot:sender
+{   NSUserDefaults  *defaults = [NSUserDefaults standardUserDefaults];
+    NSArray         *snapArray = [defaults objectForKey:@"snapShotDocuments"];
+    int             i;
+
+    for (i=0; i<[snapArray count]; i++)
+    {   NSArray *array = [snapArray objectAtIndex:i];
+
+        if ( [array isKindOfClass:[NSString class]] )
+            [self openFile:[snapArray objectAtIndex:i]];
+        else if ([array count] >= 1)
+        {   [self openFile:[array objectAtIndex:0]];
+            if (document && [array count] >= 2) // point or rect
+            {    NSArray    *components = [[array objectAtIndex:1] componentsSeparatedByString:@" "];
+
+                if ( [components count] >= 4 )       // rectangle
+                {   NSRect  rect = rectFromPropertyList([array objectAtIndex:1]);
+
+                    [[document window] setFrame:rect display:NO];
+#ifdef __APPLE__
+                    if ( [array count] >= 3 &&  // unfoldedSize
+                        [[array objectAtIndex:2] respondsToSelector:@selector(floatValue)] )
+                    {   float   h = [[array objectAtIndex:2] floatValue];
+                        [[document window] setUnfoldedHeight:h];
+                    }
+#endif
+                }
+                else                            // point only (v 3.9.1 only)
+                {   NSPoint p = pointFromPropertyList([array objectAtIndex:1]);
+
+                    [[document window] setFrameOrigin:p];
+                }
+            }
+        }
+    }
+}
+
 /*
  * Methods to load model files for the various panels.
  */
@@ -867,13 +937,39 @@ typedef enum
 
     [serialNumber setStringValue:@""];
 
-    /*{   NSDictionary    *infoDict = [[NSBundle mainBundle] infoDictionary];
-        NSString    *version;
+    /* set version number and date of compilation */
+    {   NSDictionary    *infoDict = [[NSBundle mainBundle] infoDictionary];
+        NSString        *version = [infoDict objectForKey:@"CFBundleVersion"];
 
-        //NSLog(@"%@", infoDict);
-        if ( (version=[infoDict objectForKey:@"CFBundleVersion"]) )
-            [infoVersionNo setStringValue:version]; // ex: 3.9.0 pre 3 (2008-03-01)
-    }*/
+        if ( version )
+        {   char    *compileDate = __DATE__;    // Apr 10 2010
+            char    date[15];
+
+            if ( strlen(compileDate) == 11 )    // Apr 10 2010 -> 2010-04-10
+            {   NSArray *mArray;
+                char    mStr[4];
+                int     m;
+
+                strncpy(mStr, compileDate, 3); mStr[3] = 0;
+                mArray = [NSArray arrayWithObjects:@"Jan", @"Feb", @"Mar", @"Apr", @"May", @"Jun", @"Jul", @"Aug", @"Sep", @"Oct", @"Nov", @"Dec", nil];
+                if ( (m = [mArray indexOfObject:[NSString stringWithUTF8String:mStr]]) != NSNotFound )
+                {   m ++;
+                    strncpy(date, compileDate+7, 4);    // YYYY
+                    date[4] = '-';
+                    date[5] = (m >= 10) ? '1' : '0';
+                    if (m >= 10) m -= 10;
+                    date[6] = '0' + m;                  // MM
+                    date[7] = '-';
+                    strncpy(date+8, compileDate+4, 2);    // DD
+                    date[10] = 0;
+                    compileDate = date;
+                }
+            }
+            if ( compileDate )
+                version = [version stringByAppendingFormat:@" (%s)", compileDate];
+            [infoVersionNo setStringValue:version]; // ex: 3.9.1 pre 1 (2010-02-13)
+        }
+    }
 
     [[NSNotificationCenter defaultCenter] postNotificationName:InfoPanelWillDisplay
                                                         object:nil userInfo:nil];
@@ -1096,19 +1192,19 @@ typedef enum
  * modified: 2004-12-03
  */
 - (void)setCurrent2DTool:sender
-{   id		cursor;
-    id		matrix = [[[toolPanel contentView] subviews] objectAtIndex:0];
-    static id	rotateCursor = nil, crossCursor = nil, scissorCursor = nil;	// the cursors
+{   id          cursor;
+    id          matrix = [[[toolPanel contentView] subviews] objectAtIndex:0];
+    static id   rotateCursor = nil, crossCursor = nil, scissorCursor = nil;	// the cursors
 
     if ( [self currentDocument] )
     {
-        if (!sender && current2DTool)
+        if (!sender && current2DTool)   // temporary arrow mode by pressing Alternate
         {   current2DTool = 0;
             [[[self currentDocument] scrollView] setDocumentCursor:[NSCursor arrowCursor]];
             return;
         }
 
-        if (sender && sender!=self)	// command key pressed
+        if (sender && sender != self)	// command key pressed
             [[[self currentDocument] window] endEditingFor:nil];	// end editing of text
         current2DTool = [[matrix selectedCell] tag];
         switch (current2DTool)
@@ -1213,7 +1309,8 @@ typedef enum
 - (BOOL)alternate	{ return alternate; }
 
 /* created:  1995-11-05
- * modified: 2009-03-27
+ * modified: 2010-02-18 (right mouse down exits editing modes)
+ *           2009-03-27
  *
  * We override this because we need to find out when the command key is down
  * and to change ',' to '.' for the decimal separator on the numeric pad
@@ -1272,7 +1369,15 @@ typedef enum
                 }
             }
         }
+        /* right mouse down - we exit editing mode */
+        if ([event type] == NSRightMouseDown && current2DTool)
+        {   id  matrix = [[[toolPanel contentView] subviews] objectAtIndex:0];
+
+            [matrix selectCellWithTag:0];
+            [self setCurrent2DTool:self];
+        }
     }
+
     [super sendEvent:event];
 }
 
