@@ -1,11 +1,12 @@
 /* Document.m
  * Cenon document class
  *
- * Copyright (C) 1996-2005 by vhf interservice GmbH
+ * Copyright (C) 1996-2008 by vhf interservice GmbH
  * Author: Georg Fleischmann
  *
  * Created:  1996-02-09
- * Modified: 2005-05-06 (save printinfo)
+ * Modified: 2008-10-21 (clean-up)
+ *           2008-07-24 (document units, update units)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the vhf Public License as
@@ -44,6 +45,9 @@
 #include "HPGLExportSub.h"
 #include "GerberExportSub.h"
 #include "DXFExportSub.h"
+#ifdef __APPLE__
+#    include "GSPropertyListSerialization.h"
+#endif
 
 #define XOFFSET		5.0 	// Offset of subsequent windows
 #define YOFFSET 	-20.0
@@ -157,24 +161,27 @@ static NSMutableArray *zoneList = nil;
  * inches.
  */
 + new
-{
-    self = [[super allocWithZone:[self newZone]] init];
-    //self = [[Document allocWithZone:[Document newZone]] init];
-    if (![NSBundle loadModelNamed:@"Document" owner:self])
+{   Document	*newDocument = nil;
+
+    newDocument = [[super allocWithZone:[self newZone]] init];
+    //newDocument = [[Document allocWithZone:[Document newZone]] init];
+    if (![NSBundle loadModelNamed:@"Document" owner:newDocument])
     {	NSLog(@"Cannot load Document model");	
-        [self release];
+        [newDocument release];
         return nil;
     }
 
-    [(Document*)self setName:nil andDirectory:nil];
-    [(Document*)self initWindow];
-
-    return self;
+    [newDocument setName:nil andDirectory:nil];
+    [newDocument initWindow];
+    return newDocument;
 }
 
 - (id)init
 {
     [super init];
+
+    baseUnit = -1;  // use unit from Preferences
+
     //[self registerForServicesMenu];
     return self;
 }
@@ -221,17 +228,17 @@ static NSMutableArray *zoneList = nil;
     return self;
 }
 
-/* modified: 2002-11-29
+/* modified: 2008-07-24 (load baseUnit, project settings)
  */
 + newFromFile:(NSString*)fileName
-{   Document		*newDocument = nil;
-    NSDictionary	*plist = nil;
-    NSString		*fileDirectory = nil, *outputFile = nil;
-    NSString		*directoryContainingDocument = nil;
-    NSFileManager	*fileManager = [NSFileManager defaultManager];
-    BOOL		isDirectory = NO;
-    DocView		*view;
-    NSRect		fRect;
+{   Document        *newDocument = nil;
+    NSDictionary    *plist = nil;
+    NSString        *fileDirectory = nil, *outputFile = nil;
+    NSString        *directoryContainingDocument = nil;
+    NSFileManager   *fileManager = [NSFileManager defaultManager];
+    BOOL            isDirectory = NO;
+    DocView         *view;
+    NSRect          fRect;
 
     if ([fileManager fileExistsAtPath:fileName isDirectory:&isDirectory])
     {
@@ -266,10 +273,17 @@ static NSMutableArray *zoneList = nil;
             [newDocument->printInfo setHorizontalPagination:NSAutoPagination];
             [newDocument->printInfo setVerticalPagination:NSAutoPagination];
 
-            newDocument->docVersion = [[plist objectForKey:@"DocVersion"] retain];
-            newDocument->docAuthor = [[plist objectForKey:@"Author"] retain];
-            newDocument->docCopyright = [[plist objectForKey:@"Copyright"] retain];
-            newDocument->docComment = [[plist objectForKey:@"Comment"] retain];
+            /* Project Info */
+            newDocument->docVersion   = [[plist objectForKey:@"DocVersion"] retain];
+            newDocument->docAuthor    = [[plist objectForKey:@"Author"]     retain];
+            newDocument->docCopyright = [[plist objectForKey:@"Copyright"]  retain];
+            newDocument->docComment   = [[plist objectForKey:@"Comment"]    retain];
+
+            /* Project Settings */
+            if ([plist objectForKey:@"DocSettings"])
+                newDocument->docSettingsDict = [[plist objectForKey:@"DocSettings"] mutableCopy];
+            if ([plist objectForKey:@"Unit"])
+                newDocument->baseUnit        = [plist intForKey:@"Unit"];
 
             directoryContainingDocument = fileDirectory ? fileDirectory
                                                         : [fileName stringByDeletingLastPathComponent];
@@ -291,6 +305,18 @@ static NSMutableArray *zoneList = nil;
             }
 
             [newDocument->window setFrameFromString:[plist objectForKey:@"WindowFrame"]];
+#ifdef __APPLE__    // needed, if a folded window is loaded to restore the window frame
+            {   float  h = [plist floatForKey:@"WindowUnfoldedHeight"];
+
+                [newDocument->window setUnfoldedHeight:h];
+                if ( h > 20.0 ) // folded window: we have to reinforce the frame height
+                {   NSRect  frame = rectFromPropertyList([plist objectForKey:@"WindowFrame"]);
+
+                    //frame.size.height = h;    // if we want to open windows unfolded (add y!)
+                    [newDocument->window setFrame:frame display:NO];
+                }
+            }
+#endif
             /* GNUSTEP: make sure window is not bigger than necessary
              *          (the window frame doesn't include title bar and resize bar)
              */
@@ -316,12 +342,12 @@ static NSMutableArray *zoneList = nil;
 
             [view setDocument:newDocument];
             newDocument->magazineIndex = [plist intForKey:@"MagazineIndex"];
-            [[NSNotificationCenter defaultCenter] postNotificationName:DocWindowDidChange
-                                                                object:newDocument];
             [newDocument->scrollView setDocumentView:view];
             [view release];
             [newDocument->scrollView setDocument:newDocument];
             [newDocument setName:fileDirectory ? fileDirectory : fileName];	// needed to load VImage
+            [[NSNotificationCenter defaultCenter] postNotificationName:DocWindowDidChange
+                                                                object:newDocument];    // moved from before -setDocumentView: to here
             [newDocument->window setDocument:newDocument];
             [newDocument->window makeFirstResponder:view];
             if ( [[plist objectForKey:@"Data"] isKindOfClass:[NSDictionary class]] )
@@ -344,7 +370,9 @@ static NSMutableArray *zoneList = nil;
                     NS_ENDHANDLER
                 }
                 else
-                    [view setAllLayerDirty:YES];
+                {   [view setAllLayerDirty:YES];
+                    [newDocument setDirty:NO];	// only layers are dirty to calculate output, not the document
+                }
             }
 
             [newDocument resetScrollers];	// ???
@@ -355,6 +383,8 @@ static NSMutableArray *zoneList = nil;
             fRect = rectFromPropertyList([plist objectForKey:@"ViewSize"]);
             [view setFrameSize:fRect.size];
 
+            if ( [plist objectForKey:@"hasCoordBox"] && [plist intForKey:@"hasCoordBox"] == 0 )
+                [newDocument->window enableCoordDisplay:NO];
             [newDocument->window makeKeyAndOrderFront:newDocument];
             [newDocument->window makeFirstResponder:view];	// removed by makeKeyAndOrderFront !
             [newDocument->window setMiniwindowImage:[NSImage imageNamed:@"typeCenon"]];
@@ -413,15 +443,16 @@ static NSMutableArray *zoneList = nil;
 
 /*
  * Creates a new document from what is in the passed stream.
+ * modified: 2008-07-19 (-init allocated document)
  */
 + newFromList:(NSMutableArray*)list
-{   NSRect		frameRect, fRect;
-    DocView		*view;
-    TileScrollView	*sView;
-    DocWindow		*win;
-    Document		*newDocument;
+{   NSRect          frameRect, fRect;
+    DocView         *view;
+    TileScrollView  *sView;
+    DocWindow       *win;
+    Document        *newDocument;
 
-    newDocument = [super allocWithZone:[self newZone]];
+    newDocument = [[super allocWithZone:[self newZone]] init];
     if (![NSBundle loadModelNamed:@"Document" owner:newDocument])
     {	NSLog(@"Cannot load Document model");	
         [newDocument release];
@@ -577,9 +608,9 @@ static NSMutableArray *zoneList = nil;
 #endif
 
 - (void)scale:(float)x :(float)y withCenter:(NSPoint)center
-{   NSPoint		p = {0, 0};
+{   NSPoint	p = {0, 0};
     DocView	*view = [scrollView documentView];
-    NSRect		oldFrame = [view frame], bRect;
+    NSRect	oldFrame = [view frame], bRect;
 
     /* scale doc view and cache window */
     [view scaleUnitSquareToSize:NSMakeSize(x, y)];
@@ -703,28 +734,97 @@ static NSMutableArray *zoneList = nil;
     docVersion = [newVersion retain];
     [self setDirty:YES];
 }
-- (NSString*)docVersion		{ return docVersion; }
+- (NSString*)docVersion     { return docVersion; }
 - (void)setDocAuthor:(NSString*)newAuthor
 {
     [docAuthor release];
     docAuthor = [newAuthor retain];
     [self setDirty:YES];
 }
-- (NSString*)docAuthor		{ return docAuthor; }
+- (NSString*)docAuthor      { return docAuthor; }
 - (void)setDocCopyright:(NSString*)newCopyright
 {
     [docCopyright release];
     docCopyright = [newCopyright retain];
     [self setDirty:YES];
 }
-- (NSString*)docCopyright	{ return docCopyright; }
+- (NSString*)docCopyright   { return docCopyright; }
 - (void)setDocComment:(NSString*)newComment
 {
     [docComment release];
     docComment = [newComment copy];
     [self setDirty:YES];
 }
-- (NSString*)docComment		{ return docComment; }
+- (NSString*)docComment     { return docComment; }
+
+/* Document setting
+ * created:  2008-07-19
+ * modified: 2008-07-24
+ */
+- (NSMutableDictionary*)docSettingsDict
+{
+    if (!docSettingsDict)
+        docSettingsDict = [[NSMutableDictionary dictionary] retain];
+    return docSettingsDict;
+}
+- (void)setBaseUnit:(CenonUnit)unit { baseUnit = unit; }
+- (CenonUnit)baseUnitFlat           { return baseUnit; }
+- (CenonUnit)baseUnit
+{   NSUserDefaults  *defaults = [NSUserDefaults standardUserDefaults];
+    return (baseUnit == -1) ? [defaults integerForKey:@"unit"] : baseUnit;
+}
+- (float)convertToUnit:(float)iValue
+{   NSUserDefaults  *defaults = [NSUserDefaults standardUserDefaults];
+    CenonUnit       unit = (baseUnit == -1) ? [defaults integerForKey:@"unit"] : baseUnit;
+
+    switch ( unit )
+    {
+        case UNIT_MM:		return (iValue * 25.4/72.0);
+        case UNIT_INCH:		return (iValue / 72.0);
+        default:
+        case UNIT_POINT:	return iValue;
+    }
+    return iValue;
+}
+- (float)convertFrUnit:(float)uValue
+{   NSUserDefaults  *defaults = [NSUserDefaults standardUserDefaults];
+    CenonUnit       unit = (baseUnit == -1) ? [defaults integerForKey:@"unit"] : baseUnit;
+
+    switch ( unit )
+    {
+        case UNIT_MM:		return (uValue / 25.4*72.0);
+        case UNIT_INCH:		return (uValue * 72.0);
+        default:
+        case UNIT_POINT:	return uValue;
+    }
+    return uValue;
+}
+- (float)convertMMToUnit:(float)mmValue
+{   NSUserDefaults  *defaults = [NSUserDefaults standardUserDefaults];
+    CenonUnit       unit = (baseUnit == -1) ? [defaults integerForKey:@"unit"] : baseUnit;
+
+    switch ( unit )
+    {
+        default:
+        case UNIT_MM:		return mmValue;
+        case UNIT_INCH:		return mmValue / 25.4;
+        case UNIT_POINT:	return mmValue * 72.0/25.4;
+    }
+    return mmValue;
+}
+- (float)convertUnitToMM:(float)uValue
+{   NSUserDefaults  *defaults = [NSUserDefaults standardUserDefaults];
+    CenonUnit       unit = (baseUnit == -1) ? [defaults integerForKey:@"unit"] : baseUnit;
+
+    switch ( unit )
+    {
+        default:
+        case UNIT_MM:		return uValue;
+        case UNIT_INCH:		return uValue * 25.4;
+        case UNIT_POINT:	return uValue / 72.0*25.4;
+    }
+    return uValue;
+}
 
 
 /*
@@ -770,29 +870,29 @@ static NSMutableArray *zoneList = nil;
 
     if ((newName && [newName length]) || !name)
     {
- 	if (!newName || ![newName length])
+        if (!newName || ![newName length])
         {
-	    newName = UNTITLED_STRING;
-	    if (untitledCount)
-		newName = [newName stringByAppendingFormat:@"%d", untitledCount];
-	    untitledCount++;
-	}
-	if (![newName isEqual:name])
+            newName = UNTITLED_STRING;
+            if (untitledCount)
+                newName = [newName stringByAppendingFormat:@"%d", untitledCount];
+            untitledCount++;
+        }
+        if (![newName isEqual:name])
         {
-	    [name autorelease];
-	    name = [newName copyWithZone:(NSZone*)[self zone]];
-	}
+            [name autorelease];
+            name = [newName copyWithZone:(NSZone*)[self zone]];
+        }
     }
 
     if ((newDirectory && [newDirectory length]) || !directory)
     {
- 	if (!newDirectory || ![newDirectory length])
-	    newDirectory = [(App*)NSApp currentDirectory];
-	if (![newDirectory isEqual:directory])
+        if (!newDirectory || ![newDirectory length])
+            newDirectory = [(App*)NSApp currentDirectory];
+        if (![newDirectory isEqual:directory])
         {
-	    [directory autorelease];
-	    directory = [newDirectory copyWithZone:(NSZone*)[self zone]];
-	}
+            [directory autorelease];
+            directory = [newDirectory copyWithZone:(NSZone*)[self zone]];
+        }
     }
 
     if (!name)
@@ -804,6 +904,9 @@ static NSMutableArray *zoneList = nil;
     {
         [window setTitleWithRepresentedFilename:fileName];
         [window setMiniwindowTitle:fileName];
+#ifdef __APPLE__    // well, he! the are lazy developers at Apple, the above works two times then never again
+        [window setTitle:name];
+#endif
         NSSetZoneName((NSZone*)[self zone], fileName);
     }
 }
@@ -815,19 +918,18 @@ static NSMutableArray *zoneList = nil;
 - (BOOL)setName:(NSString *)file
 {
     if (file)
-    {
-    	NSString *lastComponent = [file lastPathComponent];
-	
-	if (![lastComponent isEqual:@""])
+    {	NSString *lastComponent = [file lastPathComponent];
+
+        if (![lastComponent isEqual:@""])
         {
-	    [self setName:lastComponent andDirectory:[file stringByDeletingLastPathComponent]];
-	    return YES;
-	}
+            [self setName:lastComponent andDirectory:[file stringByDeletingLastPathComponent]];
+            return YES;
+        }
         else
         {
-	    [self setName:file andDirectory:nil];
-	    return YES;
-	}
+            [self setName:file andDirectory:nil];
+            return YES;
+        }
     }
 
     return NO;
@@ -837,7 +939,7 @@ static NSMutableArray *zoneList = nil;
 {
     [window setTitle:title];
     haveSavedDocument = NO;
-    [self setName:title andDirectory:vhfHomeDirectory()]; 
+    [self setName:title andDirectory:NSHomeDirectory()];
 }
 
 - (void)setFontObject:(Type1Font*)fontObj
@@ -927,17 +1029,19 @@ static NSMutableArray *zoneList = nil;
  * 2. remove backup (fileName.cenon~)
  * 3. move file name (old) file to backup (fileName.cenon -> fileName.cenon~)
  * 4. move temporary file to file name (#fileName.cenon -> fileName.cenon)
+ *
+ * modified: 2007-07-24 (save baseUnit and project settings)
  */
 - (BOOL)save
-{   BOOL		savedOk = NO;
-    NSString		*filename = [self filename];
-    NSString		*backupFilename = [filename stringByAppendingString:@"~"];
-    //NSString		*tmpFilename = [@"#" stringByAppendingString:filename];
-    NSString		*fileDirectory = nil;
-    NSFileManager	*fileManager = [NSFileManager defaultManager];
-    NSMutableDictionary	*fileContents;
-    BOOL		isDirectory = YES;
-    NSAutoreleasePool	*pool;
+{   BOOL                savedOk = NO;
+    NSString            *filename = [self filename];
+    NSString            *backupFilename = [filename stringByAppendingString:@"~"];
+    //NSString          *tmpFilename = [@"#" stringByAppendingString:filename];
+    NSString            *fileDirectory = nil;
+    NSFileManager       *fileManager = [NSFileManager defaultManager];
+    NSMutableDictionary *fileContents;
+    BOOL                isDirectory = YES;
+    NSAutoreleasePool   *pool;
 
     /* file not writable */
     if ( [fileManager fileExistsAtPath:filename] && ![fileManager isWritableFileAtPath:filename] )
@@ -967,30 +1071,87 @@ static NSMutableArray *zoneList = nil;
             [fileContents setObject:[NSString stringWithFormat:@"%d", DOCUMENT_VERSION] forKey:@"Version"];
             [fileContents setObject:propertyListFromNSPrintInfo(printInfo) forKey:@"PrintInfo"];
             [fileContents setObject:[window stringWithSavedFrame] forKey:@"WindowFrame"];
+#ifdef __APPLE__    // needed, if a folded window is saved, to restore the window frame
+            if ( [window isFolded] )
+                [fileContents setObject:propertyListFromFloat([window unfoldedHeight])
+                                 forKey:@"WindowUnfoldedHeight"];
+#endif
             [fileContents setObject:propertyListFromNSRect([[self documentView] bounds]) forKey:@"ViewSize"];
+            [fileContents setInt:[window hasCoordDisplay] forKey:@"hasCoordBox"];
             [fileContents setInt:magazineIndex forKey:@"MagazineIndex"];
             //[fileContents setObject:propertyListFromInt([VGraphic currentGraphicIdentifier]) forKey:@"GraphicIdentifier"];
             [fileContents setObject:@"Cenon" forKey:@"Creator"];
             [fileContents setObject:[[NSCalendarDate date] descriptionWithCalendarFormat:@"%Y-%m-%d %H:%M"] forKey:@"CreationDate"];
+            /* Project Info */
             if (docVersion)
-                [fileContents setObject:docVersion forKey:@"DocVersion"];
+                [fileContents setObject:docVersion      forKey:@"DocVersion"];
             if (docAuthor)
-                [fileContents setObject:docAuthor forKey:@"Author"];
+                [fileContents setObject:docAuthor       forKey:@"Author"];
             if (docCopyright)
-                [fileContents setObject:docCopyright forKey:@"Copyright"];
+                [fileContents setObject:docCopyright    forKey:@"Copyright"];
             if (docComment)
-                [fileContents setObject:docComment forKey:@"Comment"];
+                [fileContents setObject:docComment      forKey:@"Comment"];
+            /* Project Settings */
+            if (baseUnit != -1)
+                [fileContents setObject:[NSString stringWithFormat:@"%d", baseUnit] forKey:@"Unit"];
+            if (docSettingsDict)
+                [fileContents setObject:docSettingsDict forKey:@"DocSettings"];
+
             if ( DOCUMENT_VERSION >= 101 )
                 [fileContents setObject:[[self documentView] propertyList] forKey:@"Data"];
             else
-            {
-                archiver = [[[NSArchiver alloc] initForWritingWithMutableData:[NSMutableData data]] autorelease];
+            {   archiver = [[[NSArchiver alloc] initForWritingWithMutableData:[NSMutableData data]] autorelease];
                 [[self documentView] encodeWithCoder:archiver];
                 [fileContents setObject:[archiver archiverData] forKey:@"Data"];
             }
 
-            /* FIXME: converting dictionary to string needs a lot lot of memory !! */
-            savedOk = [fileContents writeToFile:filename atomically:YES];
+#ifdef __APPLE__
+            /* ASCII PropertyList... it just doesn't work on Mac OS X,
+             * so we use the class from the GNUstep project so long
+             */
+            if ( Prefs_OSPropertyList ) // ASCII PropertyList (most human readable + half the size of XML)
+            {
+                if ( [NSPropertyListSerialization propertyList:fileContents
+                                              isValidForFormat:NSPropertyListOpenStepFormat] )
+                {   NSData      *data;
+                    NSString    *error = nil;
+                    //NSDictionary *testplist = [NSDictionary dictionary];  // not even this works!
+
+                    data = [NSPropertyListSerialization dataFromPropertyList:fileContents
+                                                                      format:NSPropertyListOpenStepFormat
+                                                            errorDescription:&error];
+                    if (error)
+                    {   NSLog(error); [error release]; }
+                    savedOk = [data writeToFile:filename atomically:YES];
+
+                    /* Apple stuff doesn't work:
+                    if ([NSPropertyListSerialization propertyList:testplist
+                                                isValidForFormat:NSPropertyListOpenStepFormat])
+                        NSLog(@"valid for ASCII");
+                    if ([NSPropertyListSerialization propertyList:testplist
+                                                 isValidForFormat:NSPropertyListXMLFormat_v1_0])
+                        NSLog(@"valid for XML");
+                    data = [NSPropertyListSerialization dataFromPropertyList:testplist
+                                                                      format:NSPropertyListOpenStepFormat
+                                                            errorDescription:&error];
+                    [data writeToFile:@"/Users/georg/Tempo/test.plist" atomically:YES];*/
+                }
+                else
+                {   NSData      *data;
+                    NSString    *error = nil;
+
+                    data = [GSPropertyListSerialization dataFromPropertyList:fileContents
+                                                                      format:NSPropertyListOpenStepFormat
+                                                            errorDescription:&error];
+                    if (error)
+                    {   NSLog(error); [error release]; }
+                    savedOk = [data writeToFile:filename atomically:YES];
+                }
+            }
+            else	// use native PropertyList format
+#endif
+                /* FIXME: converting dictionary to string needs a lot lot of memory !! */
+                savedOk = [fileContents writeToFile:filename atomically:YES];
             haveSavedDocument = savedOk;
 
             if ( fileDirectory && savedOk )
@@ -1139,18 +1300,21 @@ static NSMutableArray *zoneList = nil;
 }
 
 - (void)printSeparation:(NSPrintOperation*)op
-{   NSColor		*sepColor;
-    NSPrintOperation	*sop=nil;
-    NSPrintInfo		*pi = [op printInfo], *spi;
-    NSMutableDictionary	*piDict = [pi dictionary];
-    NSAutoreleasePool	*pool0 = [NSAutoreleasePool new], *pool;
-    NSString		*pStr=nil, *npStr=nil, *pExt=nil, *mStr, *paperName = [pi paperName];
-    NSMutableArray	*ml, *mlist=[NSMutableArray array], *layerList;
-    NSRect		mBounds, pageRect = NSZeroRect, newPageRect = NSZeroRect, oldFrame, visibleRect;
-    NSPoint		mPosition, mOffset, lr, ll, ur, ul, center, dm, pOffset = {55.0, 55.0};
-    int			i, orientation = [pi orientation];
-    DocView		*docView = [self documentView];
-    float		width, height, scale = [docView scaleFactor];
+{   NSColor             *sepColor;
+    NSPrintOperation    *sop=nil;
+    NSPrintInfo         *pi = [op printInfo], *spi;
+    NSMutableDictionary *piDict = [pi dictionary];
+    NSAutoreleasePool   *pool0 = [NSAutoreleasePool new], *pool;
+    NSString            *cropPath;
+    NSString            *pStr=nil, *npStr=nil, *pExt=nil, *mStr, *paperName = [pi paperName];
+    NSMutableArray      *ml, *mlist=[NSMutableArray array], *layerList;
+    NSRect              mBounds, pageRect = NSZeroRect, newPageRect = NSZeroRect, oldFrame, visibleRect;
+    NSPoint             mPosition = NSZeroPoint, mOffset, lr, ll, ur, ul, center, dm, pOffset = {55.0, 55.0};
+    int                 i, orientation = [pi orientation];
+    DocView             *docView = [self documentView];
+    float               width, height, scale = [docView scaleFactor];
+
+    cropPath = vhfPathWithPathComponents([[NSBundle mainBundle] resourcePath], CROPMARK_FOLDER, nil);
 
     if ([piDict objectForKey:NSPrintSavePath])
     {
@@ -1189,7 +1353,7 @@ static NSMutableArray *zoneList = nil;
     [docView setFrameSize:NSMakeSize(width, height)];
 
     /* up left - get separation mark */
-    mStr = [NSString stringWithFormat:@"%@/%@/leftup.cenon", [[NSBundle mainBundle] resourcePath], CROPMARK_FOLDER];
+    mStr = vhfPathWithPathComponents(cropPath, @"leftup.cenon", nil);
     ml = [Document listFromFile:mStr];
     /* get mark position */
     for (i=[ml count]-1; i>=0; i--)
@@ -1211,9 +1375,9 @@ static NSMutableArray *zoneList = nil;
         [mlist addObject:[ml objectAtIndex:i]];
 
     /* down right - (upleft 180 rotatet) get separation mark */
-    mStr = [NSString stringWithFormat:@"%@/%@/leftup.cenon", [[NSBundle mainBundle] resourcePath], CROPMARK_FOLDER];
+    mStr = vhfPathWithPathComponents(cropPath, @"leftup.cenon", nil);
     ml = [Document listFromFile:mStr];
-    mBounds = [docView boundsOfArray:ml];
+    mBounds = [docView boundsOfArray:ml withKnobs:NO];
     center.x = mBounds.origin.x + mBounds.size.width/2.0;
     center.y = mBounds.origin.y + mBounds.size.height/2.0;
     /* rotate around 180 */
@@ -1239,7 +1403,7 @@ static NSMutableArray *zoneList = nil;
         [mlist addObject:[ml objectAtIndex:i]];
 
     /* right up - get separation mark */
-    mStr = [NSString stringWithFormat:@"%@/%@/rightup.cenon", [[NSBundle mainBundle] resourcePath], CROPMARK_FOLDER];
+    mStr = vhfPathWithPathComponents(cropPath, @"rightup.cenon", nil);
     ml = [Document listFromFile:mStr];
     /* get mark position */
     for (i=[ml count]-1; i>=0; i--)
@@ -1261,9 +1425,9 @@ static NSMutableArray *zoneList = nil;
         [mlist addObject:[ml objectAtIndex:i]];
 
     /* down left - (right up rotate with 180) get separation mark */
-    mStr = [NSString stringWithFormat:@"%@/%@/rightup.cenon", [[NSBundle mainBundle] resourcePath], CROPMARK_FOLDER];
+    mStr = vhfPathWithPathComponents(cropPath, @"rightup.cenon", nil);
     ml = [Document listFromFile:mStr];
-    mBounds = [docView boundsOfArray:ml];
+    mBounds = [docView boundsOfArray:ml withKnobs:NO];
     center.x = mBounds.origin.x + mBounds.size.width/2.0;
     center.y = mBounds.origin.y + mBounds.size.height/2.0;
     /* rotate around 180 */
@@ -1289,7 +1453,7 @@ static NSMutableArray *zoneList = nil;
         [mlist addObject:[ml objectAtIndex:i]];
 
     /* down middle */
-    mStr = [NSString stringWithFormat:@"%@/%@/middledown.cenon", [[NSBundle mainBundle] resourcePath], CROPMARK_FOLDER];
+    mStr = vhfPathWithPathComponents(cropPath, @"middledown.cenon", nil);
     ml = [Document listFromFile:mStr];
     /* get mark position */
     for (i=[ml count]-1; i>=0; i--)
@@ -1313,9 +1477,9 @@ static NSMutableArray *zoneList = nil;
         [mlist addObject:[ml objectAtIndex:i]];
 
     /* right middle */
-    mStr = [NSString stringWithFormat:@"%@/%@/middledown.cenon", [[NSBundle mainBundle] resourcePath], CROPMARK_FOLDER];
+    mStr = vhfPathWithPathComponents(cropPath, @"middledown.cenon", nil);
     ml = [Document listFromFile:mStr];
-    mBounds = [docView boundsOfArray:ml];
+    mBounds = [docView boundsOfArray:ml withKnobs:NO];
     center.x = mBounds.origin.x + mBounds.size.width/2.0;
     center.y = mBounds.origin.y + mBounds.size.height/2.0;
     /* rotate around 180 */
@@ -1343,9 +1507,9 @@ static NSMutableArray *zoneList = nil;
         [mlist addObject:[ml objectAtIndex:i]];
 
     /* left middle */
-    mStr = [NSString stringWithFormat:@"%@/%@/middledown.cenon", [[NSBundle mainBundle] resourcePath], CROPMARK_FOLDER];
+    mStr = vhfPathWithPathComponents(cropPath, @"middledown.cenon", nil);
     ml = [Document listFromFile:mStr];
-    mBounds = [docView boundsOfArray:ml];
+    mBounds = [docView boundsOfArray:ml withKnobs:NO];
     center.x = mBounds.origin.x + mBounds.size.width/2.0;
     center.y = mBounds.origin.y + mBounds.size.height/2.0;
     /* rotate around 180 */
@@ -1547,15 +1711,15 @@ static NSMutableArray *zoneList = nil;
 }
 
 /*
- * modified: 2005-05-06
+ * modified: 2005-09-01
  */
 - (void)printDocument:sender
-{   NSPrintOperation	*op;
-    NSPrintPanel	*pp;
-    NSPrintInfo		*pi;
-    DocView		*docView = [self documentView];
-    NSRect		visibleRect, oldFrame;
-    float		width, height, scale = [docView scaleFactor];
+{   NSPrintOperation    *op;
+    NSPrintPanel        *pp;
+    NSPrintInfo         *pi;
+    DocView             *docView = [self documentView];
+    NSRect              visibleRect, oldFrame;
+    float               width, height, scale = [docView scaleFactor];
 
     /* set scale to 1.0 */
     scale = [docView scaleFactor];
@@ -1567,41 +1731,54 @@ static NSMutableArray *zoneList = nil;
     [docView scaleUnitSquareToSize:NSMakeSize(1.0/scale, 1.0/scale)];
     [docView setFrameSize:NSMakeSize(width, height)];
 
-    /* FIXME: GNUstep printing needs to be fixed for the code below */
-#ifdef GNUSTEP_BASE_VERSION
-    op = [NSPrintOperation printOperationWithView:[self documentView] printInfo:printInfo];
-    [op setShowPanels:YES];
-    [op runOperation];
-#else
-    if (![printInfo printer] && [NSPrintInfo defaultPrinter])
-        [printInfo setPrinter:[NSPrintInfo defaultPrinter]];	// set default printer for the first time
-
-    op = [NSPrintOperation printOperationWithView:[self documentView] printInfo:printInfo];
-    [op setShowPanels:YES];
-    [NSPrintOperation setCurrentOperation:op];
-
-    pp = [NSPrintPanel printPanel];
-    [pp setAccessoryView:[[(App*)NSApp printPanelAccessory] retain]];
-    [op setPrintPanel:pp];
-
-    if ([pp runModal] == NSCancelButton)
+    /* multi page document */
+    if ([docView isMultiPage])
     {
-        [op setShowPanels:NO];	// else the user get a second printpanel
-        [op runOperation];	// this will end the operation !
-    }
-    else if (![[(App*)NSApp ppaRadio] selectedRow])		// composite
-    {
-        [op setShowPanels:NO];	// else the user get a second printpanel
+        op = [NSPrintOperation printOperationWithView:[self documentView] printInfo:printInfo];
+        [op setShowPanels:YES];
         [op runOperation];
+
+        // FIXME: restore enabled/disabled pages
     }
-    else							// color separation
-    {   [[(App*)NSApp ppaRadio] selectCellAtRow:0 column:0];	// 0 is composite 
-        [self printSeparation:op];
-    }
-    pi = [op printInfo];
-    [printInfo release];
-    printInfo = [pi retain];	// remember printInfo values for the next time ! (2005-05-06)
+    /* single page document */
+    else
+    {
+        /* FIXME: GNUstep printing needs to be fixed to work with the code below */
+#ifdef GNUSTEP_BASE_VERSION
+        op = [NSPrintOperation printOperationWithView:[self documentView] printInfo:printInfo];
+        [op setShowPanels:YES];
+        [op runOperation];
+#else
+        if (![printInfo printer] && [NSPrintInfo defaultPrinter])
+            [printInfo setPrinter:[NSPrintInfo defaultPrinter]];	// set default printer for the first time
+
+        op = [NSPrintOperation printOperationWithView:[self documentView] printInfo:printInfo];
+        [op setShowPanels:YES];
+        [NSPrintOperation setCurrentOperation:op];
+
+        pp = [NSPrintPanel printPanel];
+        [pp setAccessoryView:[[(App*)NSApp printPanelAccessory] retain]];
+        [op setPrintPanel:pp];
+
+        if ([pp runModal] == NSCancelButton)
+        {
+            [op setShowPanels:NO];	// else the user get a second printpanel
+            [op runOperation];	// this will end the operation !
+        }
+        else if (![[(App*)NSApp ppaRadio] selectedRow])		// composite
+        {
+            [op setShowPanels:NO];	// else the user get a second printpanel
+            [op runOperation];
+        }
+        else							// color separation
+        {   [[(App*)NSApp ppaRadio] selectCellAtRow:0 column:0];	// 0 is composite
+            [self printSeparation:op];
+        }
+        pi = [op printInfo];
+        [printInfo release];
+        printInfo = [pi retain];	// remember printInfo values for the next time ! (2005-05-06)
 #endif
+    }
 
     /* scale view back to old scale */
     [docView scaleUnitSquareToSize:NSMakeSize(scale, scale)];
@@ -1696,6 +1873,8 @@ static NSMutableArray *zoneList = nil;
 }
 - (void)windowDidResignKey:(NSNotification *)notification
 {   [window windowDidResignKey:notification]; }
+- (void)windowDidBecomeKey:(NSNotification *)notification
+{   [window windowDidBecomeKey:notification]; }
 - (void)windowDidBecomeMain:(NSNotification *)notification
 {   [window windowDidBecomeMain:notification]; }
 - (void)windowDidResignMain:(NSNotification *)notification
@@ -1711,6 +1890,7 @@ static NSMutableArray *zoneList = nil;
         [self save];*/
     [printInfo release];
     [fontObject release];
+    [docSettingsDict release];
     [super dealloc];
 }
 
