@@ -1,11 +1,11 @@
 /* VLine.m
  * 2-D Line object
  *
- * Copyright (C) 1996-2006 by vhf interservice GmbH
+ * Copyright (C) 1996-2010 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1996-01-29
- * modified: 2006-01-17
+ * modified: 2010-02-18 (exit with right mouse click added)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the vhf Public License as
@@ -119,7 +119,10 @@
     return line;
 }
 
-#define CREATEEVENTMASK NSLeftMouseDraggedMask|NSLeftMouseDownMask|NSLeftMouseUpMask|NSPeriodicMask
+/* create
+ * modified: 2010-02-18 (exit with right mouse click)
+ */
+#define CREATEEVENTMASK NSLeftMouseDraggedMask|NSLeftMouseDownMask|NSLeftMouseUpMask|NSRightMouseDownMask|NSPeriodicMask
 - (BOOL)create:(NSEvent *)event in:(DocView*)view
 {   NSRect		viewBounds, gridBounds, drawBounds;
     NSPoint		start, last, gridPoint, drawPoint, lastPoint, hitPoint;
@@ -159,6 +162,7 @@
     while ( ((Diff(p0.x, p1.x) <= grid && Diff(p0.y, p1.y) <= grid) ||
              (!dragging && [event type] != NSLeftMouseDown) ||
               (dragging && [event type] != NSLeftMouseUp))
+            && [event type] != NSRightMouseDown
             && [event type] != NSAppKitDefined && [event type] != NSSystemDefined &&
             !([event type] == NSLeftMouseDown && [event clickCount] > 1) )
     {
@@ -258,7 +262,7 @@
                 [window postEvent:event atStart:1];	// down
         }
         /* selection of last line is done in mouseDown: by hit of nonselected objects */
-	[view display];
+        [view display];
         return NO;
     }
 
