@@ -1,11 +1,11 @@
 /* VMark.m
  * Drill marker or any other marking
  *
- * Copyright (C) 1996-2007 by vhf interservice GmbH
+ * Copyright (C) 1996-2009 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1997-11-13
- * modified: 2005-08-21
+ * modified: 2009-12-25 (description added)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the vhf Public License as
@@ -73,6 +73,11 @@
 {
 }
 
+- (NSString*)description
+{
+    return [NSString stringWithFormat:@"%@: origin = {%.4f, %.4f, %.4f}, dia = %.4f", [self title], origin.x, origin.y, z, diameter];
+}
+
 - (NSString*)title                  { return @"Mark"; }
 
 - (void)setDiameter:(float)dia      { diameter = dia; }
