@@ -1,11 +1,12 @@
 /* VRectangle.m
  * 2-D Rectangle object
  *
- * Copyright (C) 1996-2005 by vhf interservice GmbH
+ * Copyright (C) 1996-2010 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1996-09-17
- * modified: 2005-09-08
+ * modified: 2010-03-03 (-copy setDirectionCCW, -pathRepresentation edges build with r, not radius)
+ *           2006-02-08
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the vhf Public License as
@@ -69,7 +70,7 @@
 {
     [self setParameter];
     fillColor = [[NSColor blackColor] retain];
-    endColor = [[NSColor blackColor] retain];
+    endColor  = [[NSColor blackColor] retain];
     graduateAngle = 0.0;
     stepWidth = 7.0;
     radialCenter = NSMakePoint(0.5, 0.5);
@@ -88,7 +89,7 @@
 {   id		rectangle = [[VRectangle allocWithZone:[self zone]] init];
 
     [rectangle setSelected:isSelected];
-    [rectangle setLocked:isLocked];
+    [rectangle setLocked:NO];
     [rectangle setFilled:filled];
     [rectangle setWidth:width];
     [rectangle setColor:color];
@@ -132,7 +133,7 @@
 #define CREATEEVENTMASK NSLeftMouseDraggedMask|NSLeftMouseDownMask|NSLeftMouseUpMask|NSPeriodicMask
 - (BOOL)create:(NSEvent *)event in:(DocView*)view
 {   NSRect	viewBounds, gridBounds, drawBounds;
-    NSPoint	start, last, gridPoint, drawPoint, rSize, p, lastPoint, hitPoint;
+    NSPoint	start, last, gridPoint, drawPoint, rSize, p, lastPoint = NSZeroPoint, hitPoint;
     id		window = [view window];
     VRectangle 	*drawRectangleGraphic;
     BOOL	ok = YES, dragging = NO, hitEdge = NO, inTimerLoop = NO;
@@ -343,6 +344,8 @@
 
 /* return a path representation of the arc
  * returns an autoreleased object
+ * modified: 2010-03-03 (edges build with r, not radius)
+ *           2005-11-22 (max radius = 1/2 size, skip zero lines)
  */
 - (VPath*)pathRepresentation
 {   VPath		*pathG = [VPath path];
@@ -360,7 +363,7 @@
     pts[2].x = origin.x + size.x; pts[2].y = origin.y + size.y;
     pts[3].x = origin.x; pts[3].y = origin.y + size.y;
     if (size.x < r*2.0 || size.y < r*2.0)
-        r = 0.0;
+        r = Min(size.x, size.y) / 2.0;
 
     for ( i=(ccw)?0:4; (ccw)?i<4:i; (ccw)?i++:i-- )
     {
@@ -369,15 +372,16 @@
         p0.y = pts[i].y + ((pts[i].y<pts[j].y) ? r : ((pts[i].y>pts[j].y) ? -r : 0));
         p1.x = pts[j].x + ((pts[i].x<pts[j].x) ? -r : ((pts[i].x>pts[j].x) ? r : 0));
         p1.y = pts[j].y + ((pts[i].y<pts[j].y) ? -r : ((pts[i].y>pts[j].y) ? r : 0));
-        line = [VLine line];
-        [line setVertices:p0 :p1];
-        [line setColor:color];
-        [[pathG list] addObject:line];
-
+        if ( r == 0.0 || p0.x != p1.x || p0.y != p1.y )
+        {   line = [VLine line];
+            [line setVertices:p0 :p1];
+            [line setColor:color];
+            [[pathG list] addObject:line];
+        }
         if ( r )
         {
-            c.x = pts[j].x + ((pts[j].x>pts[0].x) ? -radius : radius);
-            c.y = pts[j].y + ((pts[j].y>pts[0].y) ? -radius : radius);
+            c.x = pts[j].x + ((pts[j].x>pts[0].x) ? -r : r);
+            c.y = pts[j].y + ((pts[j].y>pts[0].y) ? -r : r);
             arc = [VArc arc];
             [arc setCenter:c start:p1 angle:a];
             [arc setColor:color];
@@ -577,6 +581,11 @@
         if (graduateAngle > 360.0)
             graduateAngle -= 360.0;
         vhfRotatePointAroundCenter(&radialCenter, NSMakePoint(0.5, 0.5), -angle);
+        if (radialCenter.x > 1.0) radialCenter.x = 1.0;
+        if (radialCenter.x < 0.0) radialCenter.x = 0.0;
+        if (radialCenter.y > 1.0) radialCenter.y = 1.0;
+        if (radialCenter.y < 0.0) radialCenter.y = 0.0;
+        graduateDirty = YES;
     }
     vhfRotatePointAroundCenter(&origin, cp, -angle);
     rotAngle -= angle;
@@ -601,8 +610,7 @@
 {
     if ( !radius && !rotAngle && filled < 2 )
     {
-        [NSBezierPath setDefaultLineWidth:width];
-        [NSBezierPath setDefaultLineCapStyle:NSRoundLineCapStyle];
+        [NSBezierPath setDefaultLineCapStyle: NSRoundLineCapStyle];
         [NSBezierPath setDefaultLineJoinStyle:NSRoundLineJoinStyle];
 
         if ( filled )
@@ -627,8 +635,9 @@
 
             [NSBezierPath fillRect:NSMakeRect(origin.x, origin.y, size.x, size.y)];
         }
-        if ( width > 0.0 || !filled )
+        if ( width > 0.0 || !filled || Diff(size.x, 0) < 500*TOLERANCE || Diff(size.y, 0) < 500*TOLERANCE )
         {   NSColor	*oldColor=nil;
+            float	defaultWidth = [NSBezierPath defaultLineWidth];
 
             if (!VHFIsDrawingToScreen() && [principal separationColor])
             {   NSColor	*sepColor = [self separationColor:color]; // get individual separation color
@@ -637,7 +646,9 @@
                 [self setColor:sepColor];
             }
             [super drawWithPrincipal:principal];	// set color
+            [NSBezierPath setDefaultLineWidth:(width > 0.0) ? width : defaultWidth];
             [NSBezierPath strokeRect:NSMakeRect(origin.x, origin.y, Max( 0.01, size.x), Max( 0.01, size.y))];
+            [NSBezierPath setDefaultLineWidth:defaultWidth];
 
             if (!VHFIsDrawingToScreen() && [principal separationColor])
             {   [self setColor:oldColor];
@@ -666,13 +677,14 @@
             graduateDirty = NO;
         }
         else if (graduateList && !graduateDirty)
-        {   int	i, gCnt = [graduateList count];
+        {   int		i, gCnt = [graduateList count];
+            BOOL	antialias = VHFAntialiasing();
 
             /* draw graduateList */
             VHFSetAntialiasing(NO);
             for (i=0; i<gCnt; i++)
                 [(VGraphic*)[graduateList objectAtIndex:i] drawWithPrincipal:principal];
-            VHFSetAntialiasing(YES);
+            if (antialias) VHFSetAntialiasing(antialias);
         }
         else
             NSLog(@"VRectangle: -drawWithPrincipal filling confusion\n");
@@ -767,25 +779,52 @@
     return bRect;
 }
 
-/* FIXME: this should flatten the rectangle if it has round corners
+/* modified: 2005-10-18 (rounded corners added)
  */
 - (NSPoint)appendToBezierPath:(NSBezierPath*)bPath currentPoint:(NSPoint)currentPoint
-{   NSPoint	p;
+{   int		i, j;
+    BOOL	ccw = [self isDirectionCCW];
+    NSPoint	p0, p1, c, pts[5];
+    float	a = (ccw) ? 90.0 : -90.0, r = radius, begAngle;
 
-    if (Diff(currentPoint.x, origin.x) > 0.01 || Diff(currentPoint.y, origin.y) > 0.01)
-        [bPath moveToPoint:origin];
-    p = NSMakePoint(origin.x + size.x, origin.y);
-    if (rotAngle != 0.0)
-	p = vhfPointRotatedAroundCenter(p, rotAngle, origin);
-    [bPath lineToPoint:p];
-    p = NSMakePoint(origin.x + size.x, origin.y + size.y);
-    if (rotAngle != 0.0)
-    	p = vhfPointRotatedAroundCenter(p, rotAngle, origin);
-    [bPath lineToPoint:p];
-    p = NSMakePoint(origin.x, origin.y + size.y);
-    if (rotAngle != 0.0)
-    	p = vhfPointRotatedAroundCenter(p, rotAngle, origin);
-    [bPath lineToPoint:p];
+    pts[0] = pts[4] = origin;
+    pts[1].x = origin.x + size.x; pts[1].y = origin.y;
+    pts[2].x = origin.x + size.x; pts[2].y = origin.y + size.y;
+    pts[3].x = origin.x; pts[3].y = origin.y + size.y;
+    if (size.x < r*2.0 || size.y < r*2.0)
+        r = 0.0;
+
+    for ( i=(ccw)?0:4; (ccw)?i<4:i; (ccw)?i++:i-- )
+    {
+        j = (ccw) ? i+1 : i-1;
+        p0.x = pts[i].x + ((pts[i].x<pts[j].x) ? r : ((pts[i].x>pts[j].x) ? -r : 0));
+        p0.y = pts[i].y + ((pts[i].y<pts[j].y) ? r : ((pts[i].y>pts[j].y) ? -r : 0));
+        p1.x = pts[j].x + ((pts[i].x<pts[j].x) ? -r : ((pts[i].x>pts[j].x) ? r : 0));
+        p1.y = pts[j].y + ((pts[i].y<pts[j].y) ? -r : ((pts[i].y>pts[j].y) ? r : 0));
+
+        if (rotAngle != 0.0)
+            p0 = vhfPointRotatedAroundCenter(p0, rotAngle, origin);
+
+        if ( (!i || i == 4) && (Diff(currentPoint.x, p0.x) > 0.01 || Diff(currentPoint.y, p0.y) > 0.01) )
+            [bPath moveToPoint:p0];
+
+        if (rotAngle != 0.0)
+            p1 = vhfPointRotatedAroundCenter(p1, rotAngle, origin);
+        [bPath lineToPoint:p1];
+
+        if ( r )
+        {
+            c.x = pts[j].x + ((pts[j].x>pts[0].x) ? -radius : radius);
+            c.y = pts[j].y + ((pts[j].y>pts[0].y) ? -radius : radius);
+            if (rotAngle != 0.0)
+                c = vhfPointRotatedAroundCenter(c, rotAngle, origin);
+
+            begAngle = vhfAngleOfPointRelativeCenter(p1, c);
+
+            [bPath appendBezierPathWithArcWithCenter:c radius:radius
+                                          startAngle:begAngle endAngle:begAngle+a clockwise:(a < 0.0)];
+        }
+    }
 
     return NSMakePoint(LARGENEG_COORD, LARGENEG_COORD);
 }
@@ -947,11 +986,11 @@
 
     switch (pt_num)
     {
+        default:    // 2008-02-13 default changed from UL to LL
         case PT_LL:	p = origin; break;
         case PT_UL:	p.x = origin.x; p.y = origin.y + size.y; break;
         case PT_LR:	p.x = origin.x + size.x; p.y = origin.y; break;
-        default:
-        case PT_UR:	p.x = origin.x + size.x; p.y = origin.y + size.y;
+        case PT_UR:	p.x = origin.x + size.x; p.y = origin.y + size.y; break;
     }
     if (rotAngle)
         vhfRotatePointAroundCenter(&p, origin, rotAngle);
@@ -980,9 +1019,8 @@
         [self getPoint:i :&p];
         knobRect.origin.x = p.x - controlsize/2.0;
         knobRect.origin.y = p.y - controlsize/2.0;
-        if (!NSIsEmptyRect(NSIntersectionRect(hitRect , knobRect)))
+        if (selectedKnob != i && !NSIsEmptyRect(NSIntersectionRect(hitRect , knobRect)))
         {   *pt = p;
-            selectedKnob = i;
             return YES;
         }
     }
@@ -1021,28 +1059,66 @@
         vhfRotatePointAroundCenter(&p, origin, -rotAngle);
     bRect.origin.x = origin.x - fuzz;
     bRect.origin.y = origin.y - fuzz;
-    bRect.size.width = size.x + 2.0*fuzz;
+    bRect.size.width  = size.x + 2.0*fuzz;
     bRect.size.height = size.y + 2.0*fuzz;
     if ( NSPointInRect(p, bRect) )
-    {   NSPoint p0, p1, p2, p3;
+    {   int     i, j;
+        BOOL    ccw = [self isDirectionCCW];
+        NSPoint p0, p1, c, pts[5];
+        float   a = (ccw) ? 90.0 : -90.0, r = radius;
+        float   begAngles[4] = {270.0, 0.0, 90.0, 180.0};
 
-        if ( filled )
+        pts[0] = pts[4] = origin;
+        pts[1].x = origin.x + size.x; pts[1].y = origin.y;
+        pts[2].x = origin.x + size.x; pts[2].y = origin.y + size.y;
+        pts[3].x = origin.x; pts[3].y = origin.y + size.y;
+        if (size.x < r*2.0 || size.y < r*2.0)
+            r = 0.0;
+
+        if ( !Prefs_SelectByBorder && filled && !r )
             return YES;
+        if ( !Prefs_SelectByBorder && filled && r )
+            return [[self pathRepresentation] hit:p fuzz:fuzz];
 
+        for ( i=(ccw)?0:4; (ccw)?i<4:i; (ccw)?i++:i-- )
+        {
+            j = (ccw) ? i+1 : i-1;
+            p0.x = pts[i].x + ((pts[i].x<pts[j].x) ? r : ((pts[i].x>pts[j].x) ? -r : 0));
+            p0.y = pts[i].y + ((pts[i].y<pts[j].y) ? r : ((pts[i].y>pts[j].y) ? -r : 0));
+            p1.x = pts[j].x + ((pts[i].x<pts[j].x) ? -r : ((pts[i].x>pts[j].x) ? r : 0));
+            p1.y = pts[j].y + ((pts[i].y<pts[j].y) ? -r : ((pts[i].y>pts[j].y) ? r : 0));
+
+            if ( sqrDistancePointLine(&p0, &p1, &p) <= fuzz*fuzz )
+                return YES;
+
+            if ( r )
+            {
+                c.x = pts[j].x + ((pts[j].x>pts[0].x) ? -radius : radius);
+                c.y = pts[j].y + ((pts[j].y>pts[0].y) ? -radius : radius);
+
+                if ( distancePointArc(p, c, r, begAngles[i], a) <= fuzz )
+                    return YES;
+            }
+        }
+
+/*
         p0 = origin;
         p1 = NSMakePoint(origin.x+size.x, origin.y);
         p2 = NSMakePoint(origin.x+size.x, origin.y+size.y);
         p3 = NSMakePoint(origin.x, origin.y+size.y);
+
+
         if ( sqrDistancePointLine(&p0, &p1, &p) <= fuzz*fuzz ||
              sqrDistancePointLine(&p1, &p2, &p) <= fuzz*fuzz ||
              sqrDistancePointLine(&p2, &p3, &p) <= fuzz*fuzz ||
              sqrDistancePointLine(&p3, &p0, &p) <= fuzz*fuzz )
             return YES;
+*/
     }
     return NO;
 }
 
-/* created: 07.04.98
+/* created: 1998-04-07
  */
 - (void)setDirectionCCW:(BOOL)ccw
 {
@@ -1063,6 +1139,8 @@
 
 /*
  * return a path representing the outline of us
+ * modified: 2010-03-03 (setDirectionCCW:)
+ *           2005-09-27
  */
 - contour:(float)w
 {
@@ -1073,12 +1151,15 @@
 
         [nr setWidth:0.0];
         [nr setColor:color];
-        [nr setRotAngle:rotAngle];	
+        [nr setRotAngle:rotAngle];
         [nr setFilled:NO];
         r = (w+width) / 2.0;	// the amount of growth
         o.x = origin.x - r;   o.y = origin.y - r;
         s.x = size.x + 2.0*r; s.y = size.y + 2.0*r;
+        if ( s.x < 0.0 || s.y < 0.0 )
+            return nil;
         o = vhfPointRotatedAroundCenter(o, rotAngle, origin);
+        [nr setDirectionCCW: isDirectionCCW];
         [nr setVertices:o :s];
         return nr;
     }
@@ -1395,9 +1476,9 @@
      */
     first = YES;
     removing = NO;
-    for ( i=startI, listCnt = [[ng list] count]; first || i != startI; i++ )
+    for ( i=startI, listCnt = [[ng list] count]; startI != -1 && (first || i != startI); i++ )
     {	id	gThis;
-        NSPoint	gPrevE;
+        NSPoint	gPrevE = NSZeroPoint;
         BOOL	currentlyRemoved = NO;
 
         i = (i >= listCnt) ? 0 : i;
@@ -1640,7 +1721,7 @@
     for (i=0; i< sPairsCnt; i++)
     {   int	sptCnt = sPairsCnts[i], eptCnt = ePairsCnts[i], sIx = 1;
         int	sIs[2], eIs[2], sI0 = -1, eI0 = -1, endI = -1;
-        NSPoint	sPts[2], ePts[2];
+        NSPoint	sPts[2], ePts[2] = {NSZeroPoint, NSZeroPoint};
 
         /* count with i and sePairsCnts to the current startIs/startPts index */
         sI0 = 0;
@@ -1675,9 +1756,9 @@
                 for (k=0; k < sPairsCnt; k++)
                 {
                     if ((startIs[si] < listCnt &&
-                        [gj identicalWith:[[ng list] objectAtIndex:startIs[si]]]) ||
-                        (endIs[si] < listCnt &&
-                         [gj identicalWith:[[ng list] objectAtIndex:endIs[si]]]))
+                        [gj identicalWith:[[ng list] objectAtIndex:startIs[si]]]))
+                         // || (endIs[si] < listCnt &&
+                         // [gj identicalWith:[[ng list] objectAtIndex:endIs[si]]])
                     {
                         [splitListUg removeObjectAtIndex:j];
                         /* korrect all uStartIs behind j */
@@ -1803,9 +1884,9 @@
                                 for (l=0; l < sPairsCnt; l++)
                                 {
                                     if ((startIs[si] < listCnt &&
-                                         [gkn identicalWith:[[ng list] objectAtIndex:startIs[si]]]) ||
-                                        (endIs[si] < listCnt &&
-                                         [gkn identicalWith:[[ng list] objectAtIndex:endIs[si]]]))
+                                         [gkn identicalWith:[[ng list] objectAtIndex:startIs[si]]]))
+                                    // || (endIs[si] < listCnt &&
+                                    // [gkn identicalWith:[[ng list] objectAtIndex:endIs[si]]])
                                     {
                                         gknIsDouble = YES;
                                         break;
@@ -1889,9 +1970,9 @@
                                 for (l=0; l < sPairsCnt; l++)
                                 {
                                     if ((startIs[si] < listCnt &&
-                                         [gkp identicalWith:[[ng list] objectAtIndex:startIs[si]]]) ||
-                                        (endIs[si] < listCnt &&
-                                         [gkp identicalWith:[[ng list] objectAtIndex:endIs[si]]]))
+                                         [gkp identicalWith:[[ng list] objectAtIndex:startIs[si]]]))
+                                    // || (endIs[si] < listCnt &&
+                                    // [gkp identicalWith:[[ng list] objectAtIndex:endIs[si]]])
                                     {
                                         gkpIsDouble = YES;
                                         break;
@@ -2223,7 +2304,7 @@
         [plist setObject:propertyListFromFloat(graduateAngle) forKey:@"graduateAngle"];
     if (stepWidth != 7)
         [plist setObject:propertyListFromFloat(stepWidth) forKey:@"stepWidth"];
-    if (radialCenter.x != 0.5 && radialCenter.y != 0.5)
+    if (!(radialCenter.x == 0.5 && radialCenter.y == 0.5))
         [plist setObject:propertyListFromNSPoint(radialCenter) forKey:@"radialCenter"];
     return plist;
 }
