@@ -1,11 +1,12 @@
 /* VText.m
  * 2-D Text object
  *
- * Copyright (C) 1996-2009 by vhf interservice GmbH
+ * Copyright (C) 1996-2010 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1996-02-15
- * modified: 2009-02-12 (-textDidEndEditing: add correction to height to make text appear safely on Apple)
+ * modified: 2009-12-29 (-drawWithPrincipal:): for GNUstep renew drawText after rotAngle
+ *           2009-02-12 (-textDidEndEditing: add correction to height to make text appear safely on Apple)
  *           2009-02-11 (-getFlattenedObjectAt: set yOffset for mirrored text)
  *           2009-01-27 (-textDidEndEditing: remove empty text)
  *
@@ -173,7 +174,8 @@ static void initClassVars()
 
 - (NSString*)description
 {
-    return [NSString stringWithFormat:@"VText: bounds=%f %f %f %f string=%@ angle=%f serial=%d fitHoricontal=%d centerVertical=%d", bounds.origin.x, bounds.origin.y, bounds.size.width, bounds.size.height, [self string], rotAngle, isSerialNumber, fitHorizontal, centerVertical];
+    return [NSString stringWithFormat:@"VText: bounds=%f %f %f %f string=%@ angle=%f serial=%d fitHoricontal=%d centerVertical=%d",
+            bounds.origin.x, bounds.origin.y, bounds.size.width, bounds.size.height, [self string], rotAngle, isSerialNumber, fitHorizontal, centerVertical];
 }
 
 - (NSString*)title
@@ -611,6 +613,11 @@ static void initClassVars()
     [fe setDelegate:self];
     [editView addSubview:fe];
 
+/* doesn't work ! #ifdef __APPLE__    // Workaround: display Single Line Fonts
+    if ( VHFAntialiasing() && [[font fontName] hasSuffix:@"-1Line"] )
+        VHFSetAntialiasing(NO);
+#endif*/
+
     //[fe display];	/* redraw because although we may have black background we edit on a white background*/
 
     /* Make it the first responder.
@@ -978,9 +985,21 @@ static void initClassVars()
  * draws the text
  */
 - (void)drawWithPrincipal:principal
-{   id			window = [principal window];
-    BOOL		saveAutoDisplay = [window isAutodisplay];
-    NSLayoutManager	*lm = [drawText layoutManager];
+{   id              window = [principal window];
+    BOOL            saveAutoDisplay = [window isAutodisplay];
+    NSLayoutManager *lm = [drawText layoutManager];
+
+#ifdef __APPLE__    // Workaround: display Single Line Fonts
+    BOOL    antialias = VHFAntialiasing();
+    if ( antialias && [[font fontName] hasSuffix:@"-1Line"] )
+        VHFSetAntialiasing(NO);
+    /*if ( NSAppKitVersionNumber < NSAppKitVersionNumber10_6  // < 10.6
+         && [[font fontName] hasSuffix:@"-1Line"] )         // Single Line Font
+    {   VPath   *path = [self getFlattenedObjectAt:NSMakePoint(0.0, 0.0) withOffset:0];
+        [path drawWithPrincipal:principal];
+        return;
+    }*/
+#endif
 
     editView = [principal editView];	// needed in flatten object
 
@@ -1067,11 +1086,23 @@ static void initClassVars()
         [drawText removeFromSuperview];
         [window setAutodisplay:saveAutoDisplay];
         [drawText scaleUnitSquareToSize:NSMakeSize(1.0, 1.0/aspectRatio)];
+#ifdef GNUSTEP_BASE_VERSION // 2009-12-29 gui 0.17.1: GNUstpe workaround, rotation screws up everything
+        if ( rotAngle != 0.0 )
+        {   [drawText release]; drawText = nil;
+            initClassVars();
+        }
+#endif
+        //[drawText setFrameRotation:0.0];
     	/*if (DrawStatus == Resizing)
     	{   [[NSColor lightGrayColor] set];
             NSFrameRect(bounds);
     	}*/
     }
+
+#ifdef __APPLE__    // Workaround: display Single Line Fonts
+    if (antialias)
+        VHFSetAntialiasing(antialias);
+#endif
 }
 
 /* extract number from data, increment this number and add it to data. Then write string to drawText
