@@ -1,11 +1,12 @@
 /* InspectorPanel.m
  * Cenon Inspector panel
  *
- * Copyright (C) 1996-2009 by vhf interservice GmbH
+ * Copyright (C) 1996-2010 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1993-10-01
- * modified: 2009-03-26 (-sendEvent: clean-up, comma-handling)
+ * modified: 2010-01-17 (-windowWillResize:, constrainFrameRect: added)
+ *           2009-03-26 (-sendEvent: clean-up, comma-handling)
  *           2008-03-17 (Accessory replaces AllText)
  *
  * This program is free software; you can redistribute it and/or
@@ -63,6 +64,7 @@
     return self;
 }
 
+
 - (void)update:sender
 {
 }
@@ -525,4 +527,31 @@
         [self loadGraphic:nil];
 }
 
+/* allow resizing the window in grid
+ * created: 2010-01-17
+ */
+- (NSSize)windowWillResize:(NSWindow*)sender toSize:(NSSize)newSize
+{   int gridSize = Prefs_WindowGrid;
+
+    if ( gridSize ) // grid size
+    {
+        newSize.width  = floor((newSize.width +gridSize/2) / gridSize) * gridSize;
+        newSize.height = floor((newSize.height+gridSize/2) / gridSize) * gridSize;
+    }
+    return newSize;
+}
+/* allow moving of window in grid
+ * created: 2010-01-17
+ */
+- (NSRect)constrainFrameRect:(NSRect)frameRect toScreen:(NSScreen *)screen
+{   int gridSize = Prefs_WindowGrid;
+
+    frameRect = [super constrainFrameRect:frameRect toScreen:screen];
+    if ( gridSize )
+    {   frameRect.origin.x = floor((frameRect.origin.x+gridSize/2) / gridSize) * gridSize;
+        frameRect.origin.y = floor((frameRect.origin.y+gridSize/2) / gridSize) * gridSize;
+    }
+    return frameRect;
+}
+
 @end
