@@ -1,11 +1,12 @@
 /* DXFImport.m
  * DXF import object
  *
- * Copyright (C) 1996-2009 by vhf interservice GmbH
+ * Copyright (C) 1996-2010 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1996-05-01
- * modified: 2007-01-30 (close PolyLine: check for start == end point)
+ * modified: 2010-04-03 (-getArc: negative 360 deg arcs work now, buildArc(): dx == 0 case)
+ *           2007-01-30 (close PolyLine: check for start == end point)
  *
  * This file is part of the vhf Import Library.
  *
@@ -1009,24 +1010,37 @@ static NSColor *getColorFromTable(NSString *str, NSArray *table)
     return YES;
 }
 
+/* Note: see page 323 of DXF book
+ * modified: 2010-04-03
+ */
 static void buildArc(NSPoint beg, NSPoint end, float a, NSPoint *ctr, float *angle)
 {   NSPoint	p, m;
-    double	l, ang, dy, r, an, h, sn;
+    double	l, ang, dx, dy, r, an, h;
 
     ang = 4.0 * RadToDeg(atan( a ));
-    l = sqrt(SqrDistPoints(beg, end));
-    h = (a*l)/2.0;
-    m.x = (beg.x+end.x)/2.0; m.y = (beg.y+end.y)/2.0;
-    dy = m.y - beg.y;
-    //if (h>0)	/* dont know if this is correct */
+    l = sqrt(SqrDistPoints(beg, end));  // length of arc from point to point
+    h = (a*l)/2.0;                      // height of arc
+    m.x = (beg.x+end.x)/2.0; m.y = (beg.y+end.y)/2.0;   // mit point between start and end
+    dx = m.x - beg.x;                   // 
+    dy = m.y - beg.y;                   // 
+    //if (h>0)	/* don't know if this is correct */
     //    ang = -ang;
 //printf("h:%f ang:%f\n", h, ang);
     /* asin(dy/(l/2)) = angle from 0 degree to l, we may use atan(2h/l) instead */
-    sn = Max( -1.0, Min( 1.0, dy / (l/2.0) ) );
-    if ( beg.x>end.x )
-        an = 180.0 - (RadToDeg(asin(sn)) - (((ang<0) ? -90.0 : 90.0)-ang/2.0));
-    else
-        an = RadToDeg(asin(sn)) + (((ang<0) ? -90.0 : 90.0)-ang/2.0);
+    if ( Abs(dx) < TOLERANCE )              // larger y component (vertical arc) (only tested for vertical case !)
+    {   double  cn = Max( -1.0, Min( 1.0, dx / (l/2.0) ) );
+        if ( beg.y > end.y )
+            an = 180.0 - (RadToDeg(acos(cn)) - (((ang<0) ? -90.0 : 90.0)-ang/2.0));
+        else
+            an = RadToDeg(acos(cn)) + (((ang<0) ? -90.0 : 90.0)-ang/2.0);
+    }
+    else                                    // larger x component
+    {   double  sn = Max( -1.0, Min( 1.0, dy / (l/2.0) ) );
+        if ( beg.x > end.x )
+            an = 180.0 - (RadToDeg(asin(sn)) - (((ang<0) ? -90.0 : 90.0)-ang/2.0));
+        else
+            an = RadToDeg(asin(sn)) + (((ang<0) ? -90.0 : 90.0)-ang/2.0);
+    }
     r = Abs(l/(2.0*sin(DegToRad(ang/2.0))));
 //NSLog(@"0 h=%f an=%f dy=%f l=%f ang=%f r=%f", h, an, dy, l, ang, r);
     p.x = beg.x + r; p.y = beg.y;
@@ -1119,7 +1133,7 @@ static void buildArc(NSPoint beg, NSPoint end, float a, NSPoint *ctr, float *ang
 }
 
 /* created:   1993-05-17
- * modified:  2004-08-04
+ * modified:  2010-04-03 (negative 360 deg arcs work now)
  *
  * purpose:   get arc
  * parameter: cp (the dxf data)
@@ -1145,11 +1159,15 @@ static void buildArc(NSPoint beg, NSPoint end, float a, NSPoint *ctr, float *ang
 
     start->x = ctr->x + r;
     start->y = ctr->y;
-    if (ba>=360.0) ba-=360.0;
-    if (ea<ba) ea+=360.0;
+    *angle = 0.0;
+    if ( Abs(ea - ba) >= 360.0 )
+        *angle = ea - ba;
+    if (ba >= 360.0) ba -= 360.0;
+    if (ea < ba)     ea += 360.0;
     vhfRotatePointAroundCenter(start, *ctr, ba);
 
-    *angle = ea - ba;
+    if ( *angle == 0.0 )
+        *angle = ea - ba;
 
     state.width = 0.0;
 
